/*
** lg: log checking/maintainance utility
**
** Copyright (C) 1997 Mark Rigby-Jones
** mrj@ox.compsoc.org.uk <*> http://www.ox.compsoc.org.uk/~mrj
** 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** If you do not have a copy of the GNU General Public License then
** you can get one by writing to the Free Software Foundation, Inc.,
** 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#define VERSION    "v2.0.5"

#define LOGFILE    "/var/admin/log"
#define TMPDIR     "/tmp"
#define TMPFILEPRE "lg"
#define CHKFILE    ".lg"
#define DEFPAGER   "less -eis"
#define DEFLINES   24
#define DEFEDITOR  "vim"
#define SYSTEMLEN  5000
#define FILELEN    5000

#include <errno.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <pwd.h>
#include <time.h>
#include <utime.h>
#include <sys/stat.h>
#include <sys/types.h>

#ifndef NO_GETOPT_LONG
#include <getopt.h>
#endif

void addentry(char *filename)
{
  char         tmpfile[FILELEN];
  char         command[SYSTEMLEN];
  char        *tmpdir = NULL;
  char        *editor = NULL;
  int          rc = 0;
  struct stat  chk_stat;

  if ((editor = getenv("VISUAL")) == NULL) {
    if ((editor = getenv("EDITOR")) == NULL) {
      editor = strdup(DEFEDITOR);
    }
  }

  tmpdir = getenv("TMPDIR");
  if (tmpdir == NULL) {
    tmpdir = strdup(TMPDIR);
  }
  sprintf(tmpfile, "%s/%sXXXXXX", tmpdir, TMPFILEPRE);
  rc = mkstemp(tmpfile); 

  if (rc < 0) {
    fprintf(stderr, "Couldn't open temporary file (%s) for log entry: %s\n",
                    tmpfile,
                    strerror(errno));
    exit(1);
  }

  close(rc);

  snprintf(command, SYSTEMLEN, "%s %s", editor, tmpfile);
  system(command);
  if (stat(tmpfile, &chk_stat) == 0 && chk_stat.st_size > 0) {
	  snprintf(command, SYSTEMLEN,
      "(echo ; echo `date` on `hostname` ; cat %s) | tr -s '\\n' >> %s",
      tmpfile, filename);
    system(command);
  }
  unlink(tmpfile);
  return;
}

void help(const char *progname)
{
  printf("Usage: %s [function] [logfile]\n"
    "Check whether a logfile (%s by default) has been changed, and\n"
    "print a brief message if it has since last read using lg. Other functions\n"
    "shown below. Note that for security reasons, you should not run lg whilst\n"
    "root - it will exit with an error message.\n\n"
    "-a, --add       append an entry to the logfile - allows editing of a\n"
    "                 temporary file which is added to the logfile together\n"
    "                 with time, date and host machine information.\n"
    "-p, --page      view the entire logfile using a pager\n"
    "-t, --tail      view the last 24 lines of the logfile\n"
    "-h, --help      print this information and exit\n"
    "-V, --version   print version information and exit\n\n"
    "If you need more info, then read the man page!\n", progname, LOGFILE);
  return;
}

int main(int argc, char **argv) {
  uid_t                uid;
  struct passwd        *pwd;
  char                 command[SYSTEMLEN];
  char                 chkfile[FILELEN];
  char                 *logfile;
  struct stat          log_stat, chk_stat;
  struct utimbuf       touch_time;
  char                 *pager;
  int                  mode, option_index;
  char                 *lines = NULL;
  int                  scrlines;
  FILE                 *chk = NULL;

#ifndef NO_GETOPT_LONG
  static struct option long_options[] = {
    {"add",     0, 0, 'a'},
    {"page",    0, 0, 'p'},
    {"tail",    0, 0, 't'},
    {"help",    0, 0, 'h'},
    {"version", 0, 0, 'V'},
    {0,         0, 0, 0  }
  };
#endif

  uid = getuid();
  pwd = getpwuid(uid);

  if (uid == 0) {
    printf("%s: cannot be run as root - please return to your normal uid!\n",
                    argv[0]);
    exit(1);
  }

  sprintf(chkfile, "%s/%s", pwd->pw_dir, CHKFILE);

  /* If chkfile (~lusername/.lg) doesn't exist - make it */
  if (stat(chkfile, &chk_stat)) {
    chk = fopen(chkfile, "w");
    if (chk != NULL) {
      fclose(chk);
      chk = NULL;
    } else {
      fprintf(stderr, "Warning, couldn't create: %s\n", chkfile);
    }
  }

  touch_time.actime = touch_time.modtime = time(NULL);

  /* Set pager / editor to users environment variables or defaults */
  if ((pager = getenv("PAGER")) == NULL) {
    pager = strdup(DEFPAGER);
  }

#ifndef NO_GETOPT_LONG
  mode = getopt_long(argc, argv, "apthV", long_options, &option_index);
#else
  mode = getopt(argc, argv, "apthV");
#endif /* NO_GETOPT_LONG */

  if (optind < argc) {
    logfile = argv[optind];
    optind++;
  } else {
    logfile = strdup(LOGFILE);
  }

  switch (mode) {
    case 'a':
      addentry(logfile);
      exit(0);
    case 'h':
      help(argv[0]);
      exit(0);
    case 'V':
      printf("lg %s by Mark Rigby-Jones & various other OU CompSoc admins.\n",
		      VERSION);
      exit(0);
    case 'p':
      utime(chkfile, &touch_time);
      snprintf(command, SYSTEMLEN, "%s %s", pager, logfile);
      system(command);
      break;
    case 't':
      utime(chkfile, &touch_time);
      lines = getenv("LINES");
      if (lines != NULL) {
        scrlines = atoi(lines) - 1;
      } else {
        scrlines = DEFLINES - 1;
      }
      snprintf(command, SYSTEMLEN, "tail -%d %s", scrlines, logfile);
      system(command);
      break;
    default:
      if (stat(logfile, &log_stat)) {
        perror("Couldn't stat logfile");
        exit(0);
      }
      if ((chk_stat.st_mtime < log_stat.st_mtime)) {
        printf("%s has been modified\n", logfile);
      }
      break;
  }
  return 0;
}
